@@ -13,7 +13,7 @@
 // limitations under the License.
 
 use bstr::ByteVec as _;
-use clap_complete::ArgValueCandidates;
+use clap_complete::ArgValueCompleter;
 use itertools::Itertools as _;
 use jj_lib::object_id::ObjectId as _;
 use jj_lib::rewrite::merge_commit_trees;
@@ -23,10 +23,10 @@ use crate::cli_util::CommandHelper;
 use crate::cli_util::RevisionArg;
 use crate::command_error::CommandError;
 use crate::complete;
-use crate::formatter::PlainTextFormatter;
 use crate::ui::Ui;
 
-/// Apply the reverse of given revisions on top of another revision
+/// (deprecated; use `revert`) Apply the reverse of given revisions on top of
+/// another revision
 ///
 /// The description of the new revisions can be customized with the
 /// `templates.backout_description` config variable.
@@ -38,7 +38,7 @@ pub(crate) struct BackoutArgs {
         long, short,
         default_value = "@",
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::all_revisions),
+        add = ArgValueCompleter::new(complete::revset_expression_all),
     )]
     revisions: Vec<RevisionArg>,
     /// The revision to apply the reverse changes on top of
@@ -48,7 +48,7 @@ pub(crate) struct BackoutArgs {
         long, short,
         default_value = "@",
         value_name = "REVSETS",
-        add = ArgValueCandidates::new(complete::all_revisions),
+        add = ArgValueCompleter::new(complete::revset_expression_all),
     )]
     destination: Vec<RevisionArg>,
 }
@@ -100,12 +100,7 @@ pub(crate) fn cmd_backout(
         to_back_out
             .into_iter()
             .map(|commit| {
-                let mut output = Vec::new();
-                template
-                    .format(&commit, &mut PlainTextFormatter::new(&mut output))
-                    .expect("write() to vec backed formatter should never fail");
-                // Template output is usually UTF-8, but it can contain file content.
-                let commit_description = output.into_string_lossy();
+                let commit_description = template.format_plain_text(&commit).into_string_lossy();
                 (commit, commit_description)
             })
             .collect_vec()
