@@ -19,7 +19,7 @@ use criterion::measurement::Measurement;
 use criterion::BatchSize;
 use criterion::BenchmarkGroup;
 use criterion::BenchmarkId;
-use jj_lib::revset::DefaultSymbolResolver;
+use jj_lib::revset::SymbolResolver;
 use jj_lib::revset::SymbolResolverExtension;
 use jj_lib::revset::UserRevsetExpression;
 
@@ -89,7 +89,7 @@ fn bench_revset<M: Measurement>(
         // Evaluate the expression without parsing/evaluating short-prefixes.
         let repo = workspace_command.repo().as_ref();
         let symbol_resolver =
-            DefaultSymbolResolver::new(repo, &([] as [Box<dyn SymbolResolverExtension>; 0]));
+            SymbolResolver::new(repo, &([] as [Box<dyn SymbolResolverExtension>; 0]));
         let resolved = expression
             .resolve_user_expression(repo, &symbol_resolver)
             .unwrap();
