@@ -15,7 +15,7 @@
 use std::fmt::Debug;
 use std::io::Write as _;
 
-use jj_lib::object_id::ObjectId;
+use jj_lib::object_id::ObjectId as _;
 use jj_lib::revset;
 use jj_lib::revset::RevsetDiagnostics;
 
@@ -29,6 +29,12 @@ use crate::ui::Ui;
 #[derive(clap::Args, Clone, Debug)]
 pub struct DebugRevsetArgs {
     revision: String,
+    /// Do not resolve and evaluate expression
+    #[arg(long)]
+    no_resolve: bool,
+    /// Do not rewrite expression to optimized form
+    #[arg(long)]
+    no_optimize: bool,
 }
 
 pub fn cmd_debug_revset(
@@ -37,7 +43,7 @@ pub fn cmd_debug_revset(
     args: &DebugRevsetArgs,
 ) -> Result<(), CommandError> {
     let workspace_command = command.workspace_helper(ui)?;
-    let workspace_ctx = workspace_command.revset_parse_context();
+    let workspace_ctx = workspace_command.env().revset_parse_context();
     let repo = workspace_command.repo().as_ref();
 
     let mut diagnostics = RevsetDiagnostics::new();
@@ -47,20 +53,32 @@ pub fn cmd_debug_revset(
     writeln!(ui.stdout(), "{expression:#?}")?;
     writeln!(ui.stdout())?;
 
+    if args.no_resolve && args.no_optimize {
+        return Ok(());
+    } else if args.no_resolve {
+        let expression = revset::optimize(expression);
+        writeln!(ui.stdout(), "-- Optimized:")?;
+        writeln!(ui.stdout(), "{expression:#?}")?;
+        writeln!(ui.stdout())?;
+        return Ok(());
+    }
+
     let symbol_resolver = revset_util::default_symbol_resolver(
         repo,
         command.revset_extensions().symbol_resolvers(),
         workspace_command.id_prefix_context(),
     );
-    let expression = expression.resolve_user_expression(repo, &symbol_resolver)?;
+    let mut expression = expression.resolve_user_expression(repo, &symbol_resolver)?;
     writeln!(ui.stdout(), "-- Resolved:")?;
     writeln!(ui.stdout(), "{expression:#?}")?;
     writeln!(ui.stdout())?;
 
-    let expression = revset::optimize(expression);
-    writeln!(ui.stdout(), "-- Optimized:")?;
-    writeln!(ui.stdout(), "{expression:#?}")?;
-    writeln!(ui.stdout())?;
+    if !args.no_optimize {
+        expression = revset::optimize(expression);
+        writeln!(ui.stdout(), "-- Optimized:")?;
+        writeln!(ui.stdout(), "{expression:#?}")?;
+        writeln!(ui.stdout())?;
+    }
 
     let backend_expression = expression.to_backend_expression(repo);
     writeln!(ui.stdout(), "-- Backend:")?;
