@@ -18,6 +18,7 @@ use itertools::Itertools as _;
 use jj_lib::copies::CopyRecords;
 use jj_lib::repo::Repo as _;
 use jj_lib::rewrite::merge_commit_trees;
+use pollster::FutureExt as _;
 use tracing::instrument;
 
 use crate::cli_util::print_unmatched_explicit_paths;
@@ -28,27 +29,30 @@ use crate::command_error::user_error_with_hint;
 use crate::command_error::CommandError;
 use crate::complete;
 use crate::diff_util::get_copy_records;
+use crate::diff_util::show_templated;
 use crate::diff_util::DiffFormatArgs;
 use crate::ui::Ui;
 
 /// Compare file contents between two revisions
 ///
-/// With the `-r` option, which is the default, shows the changes compared to
-/// the parent revision. If there are several parent revisions (i.e., the given
-/// revision is a merge), then they will be merged and the changes from the
-/// result to the given revision will be shown.
+/// With the `-r` option, shows the changes compared to the parent revision.
+/// If there are several parent revisions (i.e., the given revision is a
+/// merge), then they will be merged and the changes from the result to the
+/// given revision will be shown.
 ///
 /// With the `--from` and/or `--to` options, shows the difference from/to the
 /// given revisions. If either is left out, it defaults to the working-copy
 /// commit. For example, `jj diff --from main` shows the changes from "main"
 /// (perhaps a bookmark name) to the working-copy commit.
+///
+/// If no option is specified, it defaults to `-r @`.
 #[derive(clap::Args, Clone, Debug)]
 #[command(mut_arg("ignore_all_space", |a| a.short('w')))]
 #[command(mut_arg("ignore_space_change", |a| a.short('b')))]
 pub(crate) struct DiffArgs {
     /// Show changes in these revisions
     ///
-    /// If there are multiple revisions, then then total diff for all of them
+    /// If there are multiple revisions, then the total diff for all of them
     /// will be shown. For example, if you have a linear chain of revisions
     /// A..D, then `jj diff -r B::D` equals `jj diff --from A --to D`. Multiple
     /// heads and/or roots are supported, but gaps in the revset are not
@@ -57,6 +61,8 @@ pub(crate) struct DiffArgs {
     /// If a revision is a merge commit, this shows changes *from* the
     /// automatic merge of the contents of all of its parents *to* the contents
     /// of the revision itself.
+    ///
+    /// If none of `-r`, `-f`, or `-t` is provided, then the default is `-r @`.
     #[arg(
         long,
         short,
@@ -66,6 +72,8 @@ pub(crate) struct DiffArgs {
     )]
     revisions: Option<Vec<RevisionArg>>,
     /// Show changes from this revision
+    ///
+    /// If none of `-r`, `-f`, or `-t` is provided, then the default is `-r @`.
     #[arg(
         long,
         short,
@@ -75,6 +83,8 @@ pub(crate) struct DiffArgs {
     )]
     from: Option<RevisionArg>,
     /// Show changes to this revision
+    ///
+    /// If none of `-r`, `-f`, or `-t` is provided, then the default is `-r @`.
     #[arg(
         long,
         short,
@@ -90,6 +100,24 @@ pub(crate) struct DiffArgs {
         add = ArgValueCompleter::new(complete::modified_revision_or_range_files),
     )]
     paths: Vec<String>,
+    /// Render each file diff entry using the given template
+    ///
+    /// All 0-argument methods of the [`TreeDiffEntry` type] are available as
+    /// keywords in the template expression. See [`jj help -k templates`] for
+    /// more information.
+    ///
+    /// [`TreeDiffEntry` type]:
+    ///     https://jj-vcs.github.io/jj/latest/templates/#treediffentry-type
+    ///
+    /// [`jj help -k templates`]:
+    ///     https://jj-vcs.github.io/jj/latest/templates/
+    #[arg(
+        long,
+        short = 'T',
+        conflicts_with_all = ["short-format", "long-format", "tool"],
+        help_heading = "Diff Formatting Options",
+    )]
+    template: Option<String>,
     #[command(flatten)]
     format: DiffFormatArgs,
 }
@@ -161,8 +189,28 @@ pub(crate) fn cmd_diff(
         }
     }
 
-    let diff_renderer = workspace_command.diff_renderer_for(&args.format)?;
+    // -T disables both short/long rendering formats, but it might be okay to
+    // enable long format if explicitly specified (assuming -T is for short or
+    // summary output.)
+    let maybe_template;
+    let diff_renderer;
+    if let Some(text) = &args.template {
+        let language = workspace_command.commit_template_language();
+        let template = workspace_command
+            .parse_template(ui, &language, text)?
+            .labeled(["diff"]);
+        maybe_template = Some(template);
+        diff_renderer = workspace_command.diff_renderer(vec![]);
+    } else {
+        maybe_template = None;
+        diff_renderer = workspace_command.diff_renderer_for(&args.format)?;
+    }
+
     ui.request_pager();
+    if let Some(template) = &maybe_template {
+        let tree_diff = from_tree.diff_stream_with_copies(&to_tree, &matcher, &copy_records);
+        show_templated(ui.stdout_formatter().as_mut(), tree_diff, template).block_on()?;
+    }
     diff_renderer.show_diff(
         ui,
         ui.stdout_formatter().as_mut(),
